@@ -184,11 +184,14 @@ sub execute_stacks {
             check_return_value($?, $log_fh, $cmd);
 
             #
-            # Pull the depth of coverage from ustacks.
+            # Pull the depth of coverage and number of loci from ustacks.
             #
             my $depthline = (grep(/^Final coverage/, @results))[0];
             my ($depth, $stdev, $max, $nreads, $pctreads) = ($depthline =~ /mean=([\d.]+); stdev=([\d.]+); max=(\d+); n_reads=(\d+)\(([\d.]+)%\)/);
-            push(@depths_of_cov, [$sample->{'file'}, $depth, $stdev, $max, $nreads, $pctreads]);
+
+            $depthline = (grep(/^Final number of stacks built/, @results))[0];
+            my ($locicnt) = ($depthline =~ /Final number of stacks built: ([\d.]+)/);
+            push(@depths_of_cov, [$sample->{'file'}, $depth, $stdev, $max, $nreads, $pctreads, $locicnt]);
         }
 
         $i++;
@@ -733,11 +736,11 @@ sub write_depths_of_cov {
     print $log_fh
         "\nBEGIN cov_per_sample\n",
         "# Depths of Coverage for Processed Samples\n",
-        "sample\tdepth of cov\tmax cov\tnumber reads incorporated\t% reads incorporated\n";
+        "sample\tloci assembled\tdepth of cov\tmax cov\tnumber reads incorporated\t% reads incorporated\n";
 
     foreach $a (@{$depths}) {
-        print STDERR  $a->[0], "; depth: ", $a->[1], "x; max: ", $a->[3], "x; number of reads: ", $a->[4], " (", $a->[5], "%)\n";
-        print $log_fh $a->[0], "\t", $a->[1], "\t", $a->[3], "\t", $a->[4], "\t", $a->[5], "\n";
+        print STDERR  $a->[0], "; loci assembled: ", $a->[6], "; depth: ", $a->[1], "x; max: ", $a->[3], "x; number of reads: ", $a->[4], " (", $a->[5], "%)\n";
+        print $log_fh $a->[0], "\t", $a->[6], "\t", $a->[1], "\t", $a->[3], "\t", $a->[4], "\t", $a->[5], "\n";
     }
 
     print $log_fh
@@ -891,7 +894,7 @@ denovo_map.pl --samples dir --popmap path --out-path dir [--paired [--rm-pcr-dup
 
   Stack assembly options:
     -M: number of mismatches allowed between stacks within individuals (for ustacks).
-    -n: number of mismatches allowed between stacks between individuals (for cstacks; default 1; suggested: set to ustacks -M).
+    -n: number of mismatches allowed between stacks between individuals (for cstacks; default: set to ustacks -M).
 
   SNP model options:
     --var-alpha: significance level at which to call variant sites (for gstacks; default: 0.05).
