@@ -35,6 +35,12 @@ struct Group {
     static const string default_name;
 };
 
+struct Contig {
+    string name;
+    string length;
+    Contig(const string &n, const string &l) : name(n), length(l) {}
+};
+    
 /*
  * MetaPopInfo
  * Class for reprensenting a metapopulation : its samples, populations,
@@ -42,8 +48,9 @@ struct Group {
  */
 class MetaPopInfo {
     vector<Sample> samples_; //n.b. Samples are sorted primarily by population index, and secondarily by name.
-    vector<Pop> pops_;
-    vector<Group> groups_;
+    vector<Pop>    pops_;
+    vector<Group>  groups_;
+    vector<Contig> contigs_; // List of contig names and lengths for file exports.
 
     map<string,size_t> sample_indexes_; // Links a name with an index in [samples_].
     map<string,size_t> pop_indexes_;
@@ -86,15 +93,18 @@ public:
     size_t n_samples() const {return samples().size();}
     const vector<Pop>& pops() const {return pops_;}
     const vector<Group>& groups() const {return groups_;}
-
+    const vector<Contig>& contigs() const {return contigs_;}
+    
     const vector<size_t>& sample_indexes_orig_order() const {return sample_indexes_orig_order_;}
 
     size_t get_sample_index(const string& name, bool must_exist=true) const;
     size_t get_pop_index(const string& name) const {return pop_indexes_.at(name);}
     size_t get_group_index(const string& name) const {return group_indexes_.at(name);}
 
+    void   add_contig(const string &ctg_name, const string &ctg_len) { this->contigs_.push_back(Contig(ctg_name, ctg_len)); }
+    
     // Work with sample IDs. (IDs unicity is not enforced.)
-    void set_sample_id(size_t index, size_t id) {samples_.at(index).id = id; sample_indexes_by_id_[id] = index;}
+    void   set_sample_id(size_t index, size_t id) {samples_.at(index).id = id; sample_indexes_by_id_[id] = index;}
     size_t get_sample_index(const size_t& id) const {return sample_indexes_by_id_.at(id);}
 
     void status(ostream &fh);
