@@ -1,6 +1,6 @@
 // -*-mode:c++; c-style:k&r; c-basic-offset:4;-*-
 //
-// Copyright 2011-2018, Julian Catchen <jcatchen@illinois.edu>
+// Copyright 2011-2021, Julian Catchen <jcatchen@illinois.edu>
 //
 // This file is part of Stacks.
 //
@@ -49,9 +49,7 @@ LocPopSum::sum_pops(const CSLocus *cloc, Datum const*const* d, const MetaPopInfo
                     bool verbose, ostream &log_fh)
 {
     uint len = strlen(cloc->con);
-    int res;
     set<int> snp_cols;
-    int incompatible_loci = 0;
 
     for (uint i = 0; i < this->_pop_cnt; i++) {
         LocSum *s = this->_per_pop[i];
@@ -66,9 +64,8 @@ LocPopSum::sum_pops(const CSLocus *cloc, Datum const*const* d, const MetaPopInfo
                 filtered = false;
         }
         if (filtered == true) {
-            for (uint k = 0; k < len; k++) {
-                s->nucs[k].filtered_site = true;
-            }
+            for (uint k = 0; k < len; k++)
+                s->nucs[k].filtered_site(true);
             continue;
         }
 
@@ -77,35 +74,19 @@ LocPopSum::sum_pops(const CSLocus *cloc, Datum const*const* d, const MetaPopInfo
         // calculate observed genotype frequencies, allele frequencies, and expected genotype frequencies.
         //
         for (uint k = 0; k < cloc->snps.size(); k++) {
-            res = this->tally_heterozygous_pos(cloc, d, s,
-                                               cloc->snps[k]->col, k, pop.first_sample, pop.last_sample);
-            //
-            // If site is incompatible (too many alleles present), log it.
-            //
-            if (res < 0) {
-                s->nucs[cloc->snps[k]->col].incompatible_site = true;
-                incompatible_loci++;
-                if (verbose)
-                    log_fh << "within_population\t"
-                           << "incompatible_locus\t"
-                           << cloc->id << "\t"
-                           << cloc->loc.chr() << "\t"
-                           << cloc->sort_bp(cloc->snps[k]->col) +1 << "\t"
-                           << cloc->snps[k]->col << "\t"
-                           << pop.name << "\n";
-                DOES_NOT_HAPPEN;
-                // @nick 2018-12-17: I don't think this should ever happen with genotypes
-                // provided by gstacks, or generally, loaded from a VCF (as diploidy is
-                // implicit in the format).
-            }
+
+            this->tally_heterozygous_pos(cloc, d, s,
+					 cloc->snps[k]->col, k, pop.first_sample, pop.last_sample);
 
             snp_cols.insert(cloc->snps[k]->col);
         }
+
         //
         // For all other fixed sites, we just need to record them.
         //
         for (uint k = 0; k < len; k++) {
             if (snp_cols.count(k)) continue;
+
             this->tally_fixed_pos(cloc, d, s, k, pop.first_sample, pop.last_sample);
         }
 
@@ -141,20 +122,11 @@ LocPopSum::tally_fixed_pos(const CSLocus *cloc, Datum const*const* d, LocSum *s,
     s->nucs[pos].loc_id   = cloc->id;
     s->nucs[pos].bp       = cloc->sort_bp(pos);
     s->nucs[pos].fixed    = true;
-    s->nucs[pos].num_indv = num_indv;
-    s->nucs[pos].alleles  = 2 * num_indv;
+    s->nucs[pos].alleles  = (uint16_t) 2 * num_indv;
+    s->nucs[pos].num_indv((uint16_t) num_indv);
 
-    if (num_indv > 0) {
-        s->nucs[pos].p        =  1.0;
-        s->nucs[pos].p_nuc    =  p_nuc;
-        s->nucs[pos].obs_hom  =  1.0;
-        s->nucs[pos].obs_het  =  0.0;
-        s->nucs[pos].exp_hom  =  1.0;
-        s->nucs[pos].exp_het  =  0.0;
-        s->nucs[pos].stat[0]  =  0.0; // pi
-        s->nucs[pos].stat[1]  = -7.0; // fis
-        s->nucs[pos].stat[2]  =  0.0; // HWE deviation
-    }
+    if (num_indv > 0)
+        s->nucs[pos].p_nuc(p_nuc);
 
     return 0;
 }
@@ -171,6 +143,12 @@ LocPopSum::tally_heterozygous_pos(const CSLocus *cloc, Datum const*const* d, Loc
     char nuc;
 
     s->nucs[pos].reset();
+
+    //
+    // Set this nucleotide position to be heterozygous.
+    //
+    s->nucs[pos].variable_site(true);
+    
     //cout << "  Calculating summary stats at het locus " << cloc->id << " position " << pos << "; snp_index: " << snp_index << "\n";
 
     //
@@ -303,9 +281,9 @@ LocPopSum::tally_heterozygous_pos(const CSLocus *cloc, Datum const*const* d, Loc
     //
     // Calculate Pi, equivalent to expected heterozygosity (exp_het)
     //
-    s->nucs[pos].stat[0] = this->pi(tot_alleles, allele_p, allele_q);
+    s->nucs[pos].pi(this->pi(tot_alleles, allele_p, allele_q));
 
-    if (s->nucs[pos].stat[0] == 0.0)
+    if (s->nucs[pos].pi() == 0.0)
         s->nucs[pos].fixed = true;
 
     double hwe_pval = 0.0;
@@ -315,7 +293,7 @@ LocPopSum::tally_heterozygous_pos(const CSLocus *cloc, Datum const*const* d, Loc
     if (calc_hwp) {
         hwe_pval = this->hwe(num_indv, allele_p, allele_q, obs_p, obs_q, obs_het);
 
-        s->nucs[pos].stat[2] = hwe_pval;
+        s->nucs[pos].stat(2, hwe_pval);
     }
 
     //
@@ -346,25 +324,25 @@ LocPopSum::tally_heterozygous_pos(const CSLocus *cloc, Datum const*const* d, Loc
     //
     s->nucs[pos].loc_id   = cloc->id;
     s->nucs[pos].bp       = cloc->sort_bp(pos);
-    s->nucs[pos].num_indv = num_indv;
-    s->nucs[pos].alleles  = tot_alleles;
-    s->nucs[pos].p        = allele_p > allele_q ? allele_p : allele_q;
-    s->nucs[pos].p_nuc    = allele_p > allele_q ? p_allele : q_allele;
-    s->nucs[pos].q_nuc    = allele_p > allele_q ? q_allele : p_allele;
-    s->nucs[pos].obs_hom  = 1 - obs_het;
-    s->nucs[pos].obs_het  = obs_het;
-    s->nucs[pos].exp_hom  = 1 - exp_het;
-    s->nucs[pos].exp_het  = exp_het;
-
+    s->nucs[pos].alleles  = (uint16_t) tot_alleles;
+    s->nucs[pos].num_indv((uint16_t) num_indv);
+    s->nucs[pos].p(allele_p > allele_q ? allele_p : allele_q);
+    s->nucs[pos].p_nuc(allele_p > allele_q ? p_allele : q_allele);
+    s->nucs[pos].q_nuc(allele_p > allele_q ? q_allele : p_allele);
+    s->nucs[pos].obs_hom(1 - obs_het);
+    s->nucs[pos].obs_het(obs_het);
+    s->nucs[pos].exp_hom(1 - exp_het);
+    s->nucs[pos].exp_het(exp_het);
+    
     //
     // Calculate F_is, the inbreeding coefficient of an individual (I) relative to the subpopulation (S).
     // F_is measures reductions in observed heterozygosity with respect to that expected under Hardy-Weinberg
     // Equilibrium.
     //   Fis = (exp_het - obs_het) / exp_het
     //
-    double fis = s->nucs[pos].pi == 0 ? -7 : (s->nucs[pos].pi - obs_het) / s->nucs[pos].pi;
+    double fis = s->nucs[pos].pi() == 0 ? -7 : (s->nucs[pos].pi() - obs_het) / s->nucs[pos].pi();
 
-    s->nucs[pos].stat[1] = fis;
+    s->nucs[pos].stat(1, fis);
 
     return 0;
 }
@@ -522,25 +500,25 @@ LocPopSum::tally_metapop(const CSLocus *cloc)
             //
             // Sum the number of individuals examined at this locus across populations.
             //
-            mp->nucs[col].num_indv += s[j]->nucs[col].num_indv;
-            mp->nucs[col].pop_cnt  += s[j]->nucs[col].num_indv > 0 ? 1 : 0;
+            mp->nucs[col].num_indv += s[j]->nucs[col].num_indv();
+            mp->nucs[col].pop_cnt  += s[j]->nucs[col].num_indv() > 0 ? 1 : 0;
         }
 
         for (uint j = 0; j < this->_pop_cnt; j++) {
             //
             // Sum the most frequent allele across populations.
             //
-            if (s[j]->nucs[col].p_nuc == mp->nucs[col].p_allele)
+            if (s[j]->nucs[col].p_nuc() == mp->nucs[col].p_allele)
                 mp->nucs[col].p_freq +=
-                    s[j]->nucs[col].p * (s[j]->nucs[col].num_indv / (double) mp->nucs[col].num_indv);
+                    s[j]->nucs[col].p() * (s[j]->nucs[col].num_indv() / (double) mp->nucs[col].num_indv);
             else
                 mp->nucs[col].p_freq +=
-                    (1 - s[j]->nucs[col].p) * (s[j]->nucs[col].num_indv / (double) mp->nucs[col].num_indv);
+                    (1 - s[j]->nucs[col].p()) * (s[j]->nucs[col].num_indv() / (double) mp->nucs[col].num_indv);
             //
             // Sum observed heterozygosity across populations.
             //
             mp->nucs[col].obs_het +=
-                s[j]->nucs[col].obs_het * (s[j]->nucs[col].num_indv / (double) mp->nucs[col].num_indv);
+                s[j]->nucs[col].obs_het() * (s[j]->nucs[col].num_indv() / (double) mp->nucs[col].num_indv);
         }
 
         //
@@ -567,13 +545,13 @@ LocPopSum::tally_metapop(const CSLocus *cloc)
 
         if (p_cnt == 1 && q_cnt > 1) {
             for (uint j = 0; j < this->_pop_cnt; j++)
-                if (s[j]->nucs[col].p_nuc == mp->nucs[col].p_allele ||
-                    s[j]->nucs[col].q_nuc == mp->nucs[col].p_allele)
+                if (s[j]->nucs[col].p_nuc() == mp->nucs[col].p_allele ||
+                    s[j]->nucs[col].q_nuc() == mp->nucs[col].p_allele)
                     variable_pop = j;
         } else if (p_cnt > 1 && q_cnt == 1) {
             for (uint j = 0; j < this->_pop_cnt; j++)
-                if (s[j]->nucs[col].p_nuc == mp->nucs[col].q_allele ||
-                    s[j]->nucs[col].q_nuc == mp->nucs[col].q_allele)
+                if (s[j]->nucs[col].p_nuc() == mp->nucs[col].q_allele ||
+                    s[j]->nucs[col].q_nuc() == mp->nucs[col].q_allele)
                     variable_pop = j;
         }
         mp->nucs[col].priv_allele = variable_pop;
@@ -597,8 +575,8 @@ LocPopSum::tally_ref_alleles(int snp_index, uint16_t &allele_cnt,
     for (uint j = 0; j < this->_pop_cnt; j++) {
         nuc[0] = 0;
         nuc[1] = 0;
-        nuc[0] = this->_per_pop[j]->nucs[snp_index].p_nuc;
-        nuc[1] = this->_per_pop[j]->nucs[snp_index].q_nuc;
+        nuc[0] = this->_per_pop[j]->nucs[snp_index].p_nuc();
+        nuc[1] = this->_per_pop[j]->nucs[snp_index].q_nuc();
 
         for (uint k = 0; k < 2; k++)
             switch(nuc[k]) {
@@ -685,8 +663,8 @@ LocPopSum::tally_ref_alleles(int snp_index, uint16_t &allele_cnt,
     for (uint j = 0; j < this->_pop_cnt; j++) {
         nuc[0] = 0;
         nuc[1] = 0;
-        nuc[0] = this->_per_pop[j]->nucs[snp_index].p_nuc;
-        nuc[1] = this->_per_pop[j]->nucs[snp_index].q_nuc;
+        nuc[0] = this->_per_pop[j]->nucs[snp_index].p_nuc();
+        nuc[1] = this->_per_pop[j]->nucs[snp_index].q_nuc();
 
         for (uint k = 0; k < 2; k++)
             if (nuc[k] != 0 && nuc[k] == p_allele)
@@ -756,8 +734,8 @@ LocPopSum::calc_hapstats(const CSLocus *cloc, const Datum **d, const MetaPopInfo
 
                 hwp->exec_locus(pops[j].first_sample, pops[j].last_sample, d, this->_hapstats_per_pop[j]->hap_cnt);
 
-                this->_hapstats_per_pop[j]->stat[2] = hwp->_p_value;
-                this->_hapstats_per_pop[j]->stat[3] = hwp->_se;
+                this->_hapstats_per_pop[j]->stat(2, hwp->_p_value);
+                this->_hapstats_per_pop[j]->stat(3, hwp->_se);
 
                 delete hwp;
             }
@@ -853,8 +831,8 @@ LocPopSum::haplotype_diversity(int start, int end, Datum const*const* d)
     gene_diversity = (n / (n - 1)) * (1 - gene_diversity);
 
     lstat->alleles = n;
-    lstat->stat[0] = gene_diversity;
-    lstat->stat[1] = hapl_diversity;
+    lstat->stat(0, gene_diversity);
+    lstat->stat(1, hapl_diversity);
     lstat->hap_cnt = haplotypes.size();
 
     // cout << "  Population " << pop_id << " has haplotype diversity (pi) of " << s[pop_index]->pi << "\n";
@@ -1038,11 +1016,8 @@ LocusDivergence::snp_divergence(const vector<LocBin *> &loci)
                         //
                         // Locus is fixed in both populations, or was only found in one population.
                         //
-                        if (pp[pos]->pi == 0) {
-                            delete pp[pos];
-                            pp[pos] = NULL;
+                        if (pp[pos] == NULL)
                             continue;
-                        }
 
                         pp[pos]->loc_id = loc->cloc->id;
                         pp[pos]->bp     = loc->cloc->sort_bp(pos);
@@ -1055,22 +1030,16 @@ LocusDivergence::snp_divergence(const vector<LocBin *> &loci)
                         //
                         switch(fst_correction) {
                         case p_value:
-                            if (pp[pos] != NULL) {
-                                pp[pos]->stat[0] = pp[pos]->fet_p < p_value_cutoff ? pp[pos]->fst : 0;
-                                pp[pos]->stat[1] = pp[pos]->fet_p < p_value_cutoff ? pp[pos]->amova_fst : 0;
-                            }
+                            pp[pos]->amova_fst( (pp[pos]->fet_p < p_value_cutoff ? pp[pos]->amova_fst() : 0) );
                             break;
-                        case no_correction:
                         case bonferroni_win:
                         case bonferroni_gen:
-                            if (pp[pos] != NULL) {
-                                pp[pos]->stat[0] = pp[pos]->fst;
-                                pp[pos]->stat[1] = pp[pos]->amova_fst;
-                            }
+                        case no_correction:
+                        default:
                             break;
                         }
 
-                        fmean += pp[pos]->amova_fst;
+                        fmean += pp[pos]->amova_fst();
                         fcnt++;
                     }
 
@@ -1093,40 +1062,34 @@ LocusDivergence::Fst(const CSLocus *cloc, const LocPopSum *s, int pop_1, int pop
     const LocSum *s_1 = s->per_pop(pop_1);
     const LocSum *s_2 = s->per_pop(pop_2);
 
-    PopPair *pair = new PopPair();
-
     //
     // If this locus only appears in one population do not calculate Fst.
     //
-    if (s_1->nucs[pos].num_indv == 0 || s_2->nucs[pos].num_indv == 0)
-        return pair;
+    if (s_1->nucs[pos].num_indv() == 0 || s_2->nucs[pos].num_indv() == 0)
+        return NULL;
 
-    //
-    // Calculate Fst at a locus, sub-population relative to that found in the entire population
-    //   Fst = 1 - (Sum_j( (n_j choose 2) * pi_j)) / (pi_all * Sum_j( (n_j choose 2) ))
-    //
-    double n_1, n_2, pi_1, pi_2;
+    double s_1_num_indv = s_1->nucs[pos].num_indv();
+    double s_2_num_indv = s_2->nucs[pos].num_indv();
+    double n_1, n_2;
 
-    n_1  = s_1->nucs[pos].num_indv * 2;
-    n_2  = s_2->nucs[pos].num_indv * 2;
-    pi_1 = s_1->nucs[pos].pi;
-    pi_2 = s_2->nucs[pos].pi;
+    n_1  = s_1_num_indv * 2;
+    n_2  = s_2_num_indv * 2;
 
-    if (pi_1 == 0 && pi_2 == 0 && s_1->nucs[pos].p_nuc == s_2->nucs[pos].p_nuc)
-        return pair;
+    if (s_1->nucs[pos].pi() == 0 &&
+        s_2->nucs[pos].pi() == 0 &&
+        s_1->nucs[pos].p_nuc() == s_2->nucs[pos].p_nuc())
+        return NULL;
 
-    //
-    // Calculate Pi over the entire pooled population.
     //
     // First, make sure this site is compatible between the two populations (no more than two alleles).
     //
     char nucs[4];
     int  ncnt[4] = {0};
 
-    nucs[0] = s_1->nucs[pos].p_nuc;
-    nucs[1] = s_1->nucs[pos].q_nuc;
-    nucs[2] = s_2->nucs[pos].p_nuc;
-    nucs[3] = s_2->nucs[pos].q_nuc;
+    nucs[0] = s_1->nucs[pos].p_nuc();
+    nucs[1] = s_1->nucs[pos].q_nuc();
+    nucs[2] = s_2->nucs[pos].p_nuc();
+    nucs[3] = s_2->nucs[pos].q_nuc();
 
     for (int i = 0; i < 4; i++)
         switch(nucs[i]) {
@@ -1149,92 +1112,46 @@ LocusDivergence::Fst(const CSLocus *cloc, const LocPopSum *s, int pop_1, int pop
         if (ncnt[i] > 0) allele_cnt++;
 
     if (allele_cnt > 2)
-        return pair;
+        return NULL;
 
-    double tot_alleles = n_1 + n_2;
-    double p_1 = round(n_1 * s_1->nucs[pos].p);
+    PopPair *pair = new PopPair();
+
+    pair->alleles = n_1 + n_2;
+    double p_1 = round(n_1 * s_1->nucs[pos].p());
     double q_1 = n_1 - p_1;
     double p_2 =
-        s_1->nucs[pos].p_nuc == s_2->nucs[pos].p_nuc ?
-        s_2->nucs[pos].p : (1 - s_2->nucs[pos].p);
+        s_1->nucs[pos].p_nuc() == s_2->nucs[pos].p_nuc() ?
+        s_2->nucs[pos].p() : (1 - s_2->nucs[pos].p());
     p_2 = round(n_2 * p_2);
     double q_2 = n_2 - p_2;
 
-    double pi_all = s->pi(tot_alleles, p_1 + p_2, q_1 + q_2);
-
-    double bcoeff_1 = s->binomial_coeff(n_1, 2);
-    double bcoeff_2 = s->binomial_coeff(n_2, 2);
-
-    double num = (bcoeff_1 * pi_1) + (bcoeff_2 * pi_2);
-    double den = pi_all * (bcoeff_1 + bcoeff_2);
-
-    double Fst = 1 - (num / den);
-
-    pair->alleles = tot_alleles;
-    pair->fst     = Fst;
-    pair->pi      = pi_all;
-
     this->fishers_exact_test(pair, p_1, q_1, p_2, q_2);
 
-    // cout << "Locus: " << locus << ", pos: " << pos << "\n"
-    //   << "    p_1.nuc: " << s_1->nucs[pos].p_nuc << "; q_1.nuc: " << s_1->nucs[pos].q_nuc
-    //   << "; p_2.nuc: " << s_2->nucs[pos].p_nuc << "; q_2.nuc: " << s_2->nucs[pos].q_nuc << "\n"
-    //   << "    Total alleles: " << tot_alleles << "; " << " s_1.p: " << s_1->nucs[pos].p
-    //   << "; s_2.p: " << s_2->nucs[pos].p << "\n"
-    //   << "    p_1: " << p_1 << "; q_1: " << q_1 << " p_2: " << p_2 << "; q_2: " << q_2 << "\n"
-    //   << "    Pi1: " << pi_1 << "; Pi2: " << pi_2 << "; PiAll: " << pi_all << "\n"
-    //   << "    N1: " << n_1 << "; N1 choose 2: " << bcoeff_1 << "\n"
-    //   << "    N2: " << n_2 << "; N2 choose 2: " << bcoeff_2 << "\n"
-    //   << "  Fst: " << Fst << "\n";
-
     //
     // Calculate Fst (corrected for different samples sizes) using an AMOVA method,
     // correcting for unequal sample sizes.
     // Derived from Weir, _Genetic Data Analysis II_, chapter 5, "F Statistics,", pp166-167.
     //
-    double p_1_freq = s_1->nucs[pos].p;
-    double q_1_freq = 1 - p_1_freq;
+    double p_1_freq = s_1->nucs[pos].p();
     double p_2_freq =
-        s_1->nucs[pos].p_nuc == s_2->nucs[pos].p_nuc ?
-        s_2->nucs[pos].p : (1 - s_2->nucs[pos].p);
-    double q_2_freq = 1 - p_2_freq;
+        s_1->nucs[pos].p_nuc() == s_2->nucs[pos].p_nuc() ?
+        s_2->nucs[pos].p() : (1 - s_2->nucs[pos].p());
 
     double p_avg_cor =
-        ( (s_1->nucs[pos].num_indv * p_1_freq) + (s_2->nucs[pos].num_indv * p_2_freq) ) /
-        ( s_1->nucs[pos].num_indv + s_2->nucs[pos].num_indv );
-    double n_avg_cor = 2 * ((s_1->nucs[pos].num_indv / 2) + (s_2->nucs[pos].num_indv / 2));
+        ( (s_1_num_indv * p_1_freq) + (s_2_num_indv * p_2_freq) ) /
+        ( s_1_num_indv + s_2_num_indv );
+    double n_avg_cor = 2 * ((s_1_num_indv / 2.0) + (s_2_num_indv / 2.0));
 
-    pair->amova_fst =
+    pair->amova_fst(
         (
-         (s_1->nucs[pos].num_indv * pow((p_1_freq - p_avg_cor), 2) +
-          s_2->nucs[pos].num_indv * pow((p_2_freq - p_avg_cor), 2))
+         (s_1_num_indv * pow((p_1_freq - p_avg_cor), 2) +
+          s_2_num_indv * pow((p_2_freq - p_avg_cor), 2))
          /
          n_avg_cor
          )
         /
-        (p_avg_cor * (1 - p_avg_cor));
-
-    if (log_fst_comp) {
-        pair->comp     = new double[18];
-        pair->comp[0]  = n_1;
-        pair->comp[1]  = n_2;
-        pair->comp[2]  = tot_alleles;
-        pair->comp[3]  = p_1;
-        pair->comp[4]  = q_1;
-        pair->comp[5]  = p_2;
-        pair->comp[6]  = q_2;
-        pair->comp[7]  = pi_1;
-        pair->comp[8]  = pi_2;
-        pair->comp[9]  = pi_all;
-        pair->comp[10] = bcoeff_1;
-        pair->comp[11] = bcoeff_2;
-        pair->comp[12] = p_1_freq;
-        pair->comp[13] = q_1_freq;
-        pair->comp[14] = p_2_freq;
-        pair->comp[15] = q_2_freq;
-        pair->comp[16] = p_avg_cor;
-        pair->comp[17] = n_avg_cor;
-    }
+        (p_avg_cor * (1 - p_avg_cor))
+                    );
 
     // //
     // // Calculate Fst using a pure parametric method (assumes allele counts are real, not
@@ -1488,15 +1405,15 @@ LocusDivergence::haplotype_divergence_pairwise(const vector<LocBin *> &loci)
                     if (h != NULL) {
                         h->pop_1   = pop_1;
                         h->pop_2   = pop_2;
-                        h->stat[4] = haplotype_d_est(d, s, subpop_ids);
-                        h->stat[5] = haplotype_dxy(d, loc->len, subpop_ids);
+                        h->stat(2, haplotype_d_est(d, s, subpop_ids));
+                        h->stat(3, haplotype_dxy(d, loc->len, subpop_ids));
 
                         h->loc_id = loc->id;
                         h->bp     = loc->sort_bp();
 
-                        pmean   += h->stat[0];
-                        fmean   += h->stat[3];
-                        dxymean += h->stat[5];
+                        pmean   += h->stat(0);
+                        fmean   += h->stat(1);
+                        dxymean += h->stat(3);
                         cnt++;
                     }
 
@@ -1550,10 +1467,10 @@ LocusDivergence::haplotype_divergence(const vector<LocBin *> &loci)
                 h = this->haplotype_amova(d, s, pop_ids);
 
             if (h != NULL) {
-                h->stat[4] = this->haplotype_d_est(d, s, pop_ids);
+                h->stat(2, this->haplotype_d_est(d, s, pop_ids));
 
-                h->loc_id  = loc->id;
-                h->bp      = loc->sort_bp();
+                h->loc_id = loc->id;
+                h->bp     = loc->sort_bp();
             }
 
             haps->at(k) = h;
@@ -1829,29 +1746,10 @@ LocusDivergence::haplotype_amova(const Datum **d, const LocSum **s, vector<int>
     h->alleles = tot_cnt;
     h->popcnt  = valid_pop_cnt;
 
-    if (log_fst_comp) {
-        h->comp = new double[15];
-        h->comp[0]  = ssd_wp;
-        h->comp[1]  = ssd_ap_wg;
-        h->comp[2]  = ssd_ag;
-        h->comp[3]  = ssd_total;
-        h->comp[4]  = msd_wp;
-        h->comp[5]  = msd_ap_wg;
-        h->comp[6]  = msd_ag;
-        h->comp[7]  = msd_total;
-        h->comp[8]  = n;
-        h->comp[9]  = n_1;
-        h->comp[10] = n_2;
-        h->comp[11] = sigma_a;
-        h->comp[12] = sigma_b;
-        h->comp[13] = sigma_c;
-        h->comp[14] = sigma_total;
-    }
-
-    h->stat[0] = phi_st;
-    h->stat[1] = phi_ct;
-    h->stat[2] = phi_sc;
-    h->stat[3] = fst_1;
+    h->stat(0, phi_st);
+    h->stat(1, fst_1);
+    h->stat(4, phi_ct);
+    h->stat(5, phi_sc);
 
     for (uint k = 0; k < loc_hap_index.size(); k++) {
         delete [] hdists[k];
@@ -2406,22 +2304,24 @@ LocusDivergence::write_summary(string path)
 
     fh.close();
 
-    cout << "\nPopulation pair divergence statistics (more in populations.fst_summary.tsv and populations.phistats_summary.tsv):\n";
-    n = 0;
-    for (uint i = 0; i < this->_mpopi->pops().size() - 1; i++) {
+    if (this->_mpopi->pops().size() > 1) {
+	cout << "\nPopulation pair divergence statistics (more in populations.fst_summary.tsv and populations.phistats_summary.tsv):\n";
+	n = 0;
+	for (uint i = 0; i < this->_mpopi->pops().size() - 1; i++) {
 
-        for (uint j = i + 1; j < this->_mpopi->pops().size(); j++) {
-            cout << "  " << this->_mpopi->pops()[i].name << "-"
-                 << this->_mpopi->pops()[j].name
-                 << ": mean Fst: "    << this->_mean_fst[n]   / this->_mean_fst_cnt[n]
-                 << "; mean Phi_st: " << this->_mean_phist[n] / this->_mean_phist_cnt[n]
-                 << "; mean Fst': "   << this->_mean_fstp[n]  / this->_mean_fstp_cnt[n]
-                 << "; mean Dxy: "    << this->_mean_dxy[n]   / this->_mean_dxy_cnt[n]
-                 << "\n";
-            n++;
-        }
+	    for (uint j = i + 1; j < this->_mpopi->pops().size(); j++) {
+		cout << "  " << this->_mpopi->pops()[i].name << "-"
+		     << this->_mpopi->pops()[j].name
+		     << ": mean Fst: "    << this->_mean_fst[n]   / this->_mean_fst_cnt[n]
+		     << "; mean Phi_st: " << this->_mean_phist[n] / this->_mean_phist_cnt[n]
+		     << "; mean Fst': "   << this->_mean_fstp[n]  / this->_mean_fstp_cnt[n]
+		     << "; mean Dxy: "    << this->_mean_dxy[n]   / this->_mean_dxy_cnt[n]
+		     << "\n";
+		n++;
+	    }
+	}
+	cout << "\n";
     }
-    cout << "\n";
 
     return 0;
 }
