@@ -131,7 +131,7 @@ public:
     {
         return (lhs.se == rhs.se && lhs.pe == rhs.pe);
     }
-    friend ofstream& operator<<(ofstream &out, const BarcodePair &bp)
+    friend ostream& operator<<(ostream &out, const BarcodePair &bp)
     {
         if (bp.pe.length() > 0)
             out << bp.se << "-" << bp.pe;
@@ -286,6 +286,41 @@ public:
     }
 };
 
+class ReadComp {
+public:
+    Seq         *s;
+    RawRead     *r;
+    BarcodePair *bc;
+
+    ReadComp(Seq *s, RawRead *r, BarcodePair *bc) {
+	this->s  = s;
+	this->r  = r;
+	this->bc = bc;
+    }
+    void destroy() {
+	delete this->s;
+	delete this->r;
+	delete this->bc;
+    }
+};
+
+class ReadCompPair {
+public:
+    Seq         *s_1;
+    Seq         *s_2;
+    RawRead     *r_1;
+    RawRead     *r_2;
+    BarcodePair *bc;
+
+    ReadCompPair(Seq *s_1, Seq *s_2, RawRead *r_1, RawRead *r_2, BarcodePair *bc) {
+	this->s_1 = s_1;
+	this->s_2 = s_2;
+	this->r_1 = r_1;
+	this->r_2 = r_2;
+	this->bc  = bc;
+    }
+};
+
 int  parse_illumina_v1(const char *);
 int  parse_illumina_v2(const char *);
 int  parse_input_record(Seq *, RawRead *);
