@@ -1,6 +1,6 @@
 // -*-mode:c++; c-style:k&r; c-basic-offset:4;-*-
 //
-// Copyright 2011-2017, Julian Catchen <jcatchen@illinois.edu>
+// Copyright 2011-2021, Julian Catchen <jcatchen@illinois.edu>
 //
 // This file is part of Stacks.
 //
@@ -64,12 +64,6 @@ int main (int argc, char* argv[]) {
 
     parse_command_line(argc, argv);
 
-    if (oligo_len_2 == 0) oligo_len_2 = oligo_len_1;
-    min_bc_size_1 = oligo_len_1;
-    max_bc_size_1 = oligo_len_1;
-    min_bc_size_2 = oligo_len_2;
-    max_bc_size_2 = oligo_len_2;
-
     //
     // If input files are gzipped, output gziped files, unless the user chooses an output type.
     //
@@ -716,7 +710,11 @@ process_paired_reads(string prefix_1, string prefix_2, map<string, long> &counte
             oligo_1 = r_1->index_bc;
             break;
         case null_index:
-            oligo_1 = r_2->index_bc;
+	    //
+	    // In the case of dual index barcodes, they appear as NNNNNN+NNNNNN in the FASTQ headers of
+	    // both reads. The parse_input_record() function will place this barcode into r_1->index_bc.
+	    //
+            oligo_1 = r_1->index_bc;
             break;
         case inline_inline:
             oligo_1 = r_1->inline_bc;
@@ -1278,6 +1276,32 @@ int parse_command_line(int argc, char* argv[]) {
         cerr << "You must specify the length of the oligo sequences (--oligo-len-1 / --oligo-len-2).\n";
         help();
     }
+    switch(barcode_type) {
+    case null_index:
+    case null_inline:
+    case inline_null:
+    case index_null:
+	if (oligo_len_1 > 0 && oligo_len_2 == 0) {
+	    min_bc_size_1 = oligo_len_1;
+	    max_bc_size_1 = oligo_len_1;    
+	} else {
+	    min_bc_size_1 = oligo_len_2;
+	    max_bc_size_1 = oligo_len_2;
+	}
+	break;
+    case inline_inline:
+    case index_index:
+    case inline_index:
+    case index_inline:
+	min_bc_size_1 = oligo_len_1;
+	max_bc_size_1 = oligo_len_1;    
+	min_bc_size_2 = oligo_len_2;
+	max_bc_size_2 = oligo_len_2;
+	break;
+    case null_null:
+    default:
+	break;
+    }
 
     return 0;
 }
