@@ -1,6 +1,6 @@
 // -*-mode:c++; c-style:k&r; c-basic-offset:4;-*-OA
 //
-// Copyright 2013-2018, Julian Catchen <jcatchen@illinois.edu>
+// Copyright 2013-2021, Julian Catchen <jcatchen@illinois.edu>
 //
 // This file is part of Stacks.
 //
@@ -70,6 +70,23 @@ Locus::Locus(const Locus &templ)
 
 }
 
+uint
+Locus::sort_bp(uint col0) const
+{
+    if (this->loc.strand == strand_plus) {
+	return this->loc.bp + col0;
+    } else {
+	// If the locus is aligned to the negative strand and longer than
+	// the contig the returned column will be negative and will wrap
+	// around as an unsigned integer.
+	if (col0 > this->loc.bp) {
+	    // cerr << "locus::sort_bp; ID: " << this->id << "; strand: " << (this->loc.strand == strand_plus? "plus" : "minus") << "; col: " << col0 << "; bp: " << this->loc.bp << "; diff: " << this->loc.bp - col0 << "\n";
+	    DOES_NOT_HAPPEN;
+	} else
+	    return this->loc.bp - col0;
+    }
+}
+
 int
 Locus::snp_index(uint col) const
 {
