@@ -106,8 +106,10 @@ class CatalogDists {
     map<size_t, size_t> _pre_snps_per_loc, _post_snps_per_loc;
     
     size_t _pop_cnt, _sample_cnt;
+    size_t _pre_total_loci;
     size_t _total_sites, _total_loci;
     size_t *_pop_order, *_samples;
+    size_t *_pre_loci_pres_per_sample,  *_pre_loci_miss_per_sample;
     size_t *_sites_pres_per_sample, *_sites_miss_per_sample;
     size_t *_loci_pres_per_sample,  *_loci_miss_per_sample;
 
@@ -118,55 +120,27 @@ public:
         this->_pop_order  = NULL; // The array order of each population.
         this->_samples    = NULL; // Which population each sample belongs to.
 
+        this->_pre_loci_pres_per_sample  = NULL; // Array of samples tallying # loci present
+        this->_pre_loci_miss_per_sample  = NULL; // Array of samples tallying # loci missing
+
         this->_sites_pres_per_sample = NULL; // Array of samples tallying # variant sites present
         this->_sites_miss_per_sample = NULL; // Array of samples tallying # variant sites missing
         this->_loci_pres_per_sample  = NULL; // Array of samples tallying # loci present
         this->_loci_miss_per_sample  = NULL; // Array of samples tallying # loci missing
 
-        this->_total_loci  = 0;
-        this->_total_sites = 0;
-    }
-    CatalogDists(const MetaPopInfo *mpopi) {
-        assert(mpopi != NULL);
-
-        this->_pop_cnt    = mpopi->pops().size();
-        this->_sample_cnt = mpopi->n_samples();
-
-        assert(this->_pop_cnt > 0);
-        assert(this->_sample_cnt > 0);
-
-        this->_pop_order = new size_t [this->_pop_cnt];
-        this->_samples   = new size_t [this->_sample_cnt];
-        this->_sites_pres_per_sample = new size_t [this->_sample_cnt];
-        this->_sites_miss_per_sample = new size_t [this->_sample_cnt];
-        this->_loci_pres_per_sample  = new size_t [this->_sample_cnt];
-        this->_loci_miss_per_sample  = new size_t [this->_sample_cnt];
-
-        memset(this->_sites_pres_per_sample, 0, this->_sample_cnt * sizeof(size_t));
-        memset(this->_sites_miss_per_sample, 0, this->_sample_cnt * sizeof(size_t));
-        memset(this->_loci_pres_per_sample,  0, this->_sample_cnt * sizeof(size_t));
-        memset(this->_loci_miss_per_sample,  0, this->_sample_cnt * sizeof(size_t));
-        
-        this->_total_loci  = 0;
-        this->_total_sites = 0;
-
-        size_t pop_sthg = 0;
-
-        for (size_t i_pop = 0; i_pop < mpopi->pops().size(); ++i_pop) {
-            const Pop& pop = mpopi->pops()[i_pop];
-
-            for (uint i = pop.first_sample; i <= pop.last_sample; i++)
-                this->_samples[i] = pop_sthg;
-            this->_pop_order[pop_sthg] = i_pop;
-
-            pop_sthg++;
-        }
+        this->_pre_total_loci = 0;
+        this->_total_loci     = 0;
+        this->_total_sites    = 0;
     }
     ~CatalogDists() {
         if (this->_pop_order != NULL)
             delete [] this->_pop_order;
         if (this->_samples != NULL)
             delete [] this->_samples;
+        if (this->_pre_loci_pres_per_sample != NULL)
+            delete [] this->_pre_loci_pres_per_sample;
+        if (this->_pre_loci_miss_per_sample != NULL)
+            delete [] this->_pre_loci_miss_per_sample;
         if (this->_sites_pres_per_sample != NULL)
             delete [] this->_sites_pres_per_sample;
         if (this->_sites_miss_per_sample != NULL)
@@ -180,7 +154,8 @@ public:
     CatalogDists(const CatalogDists &) = delete;
     CatalogDists(CatalogDists &&) = delete;
 
-    int accumulate_pre_filtering(const size_t, const CSLocus *);
+    int init(const MetaPopInfo *mpopi);
+    int accumulate_pre_filtering(const LocBin *);
     int accumulate(const vector<LocBin *> &);
     int write_results(ostream &log_fh, const MetaPopInfo *mpopi);
 };
@@ -332,15 +307,15 @@ public:
     BatchLocusProcessor():
 	_sig_hwe_dev(NULL),
         _input_mode(InputMode::stacks2), _user_supplied_whitelist(false), _batch_size(0), _batch_num(0),
-        _mpopi(NULL), _next_loc(NULL), _unordered_bp(0) {}
+        _mpopi(NULL), _next_loc(NULL), _dists(), _unordered_bp(0) {}
     BatchLocusProcessor(InputMode mode, size_t batch_size, MetaPopInfo *popi):
 	_sig_hwe_dev(NULL),
         _input_mode(mode), _user_supplied_whitelist(false), _batch_size(batch_size), _batch_num(0),
-        _mpopi(popi), _next_loc(NULL), _dists(popi), _unordered_bp(0) {}
+        _mpopi(popi), _next_loc(NULL), _dists(), _unordered_bp(0) {}
     BatchLocusProcessor(InputMode mode, size_t batch_size):
 	_sig_hwe_dev(NULL),
         _input_mode(mode), _user_supplied_whitelist(false), _batch_size(batch_size), _batch_num(0),
-        _mpopi(NULL), _next_loc(NULL), _unordered_bp(0) {}
+        _mpopi(NULL), _next_loc(NULL), _dists(), _unordered_bp(0) {}
     ~BatchLocusProcessor() {
         for (uint i = 0; i < this->_loci.size(); i++)
             delete this->_loci[i];
@@ -428,10 +403,8 @@ int     tabulate_haplotypes(map<int, CSLocus *> &, PopMap<CSLocus> *);
 int     tabulate_locus_haplotypes(CSLocus *, Datum **, int);
 int     create_genotype_map(CSLocus *, Datum **, int);
 int     call_population_genotypes(CSLocus *, Datum **, int);
-int     translate_genotypes(map<string, string> &, map<string, map<string, string> > &, map<int, CSLocus *> &, PopMap<CSLocus> *, map<int, string> &, set<int> &); // This function doesn't exist (March 24, 2016)
+
 int     correct_fst_bonferroni_win(vector<PopPair *> &);
-int     bootstrap_fst_approximate_dist(vector<double> &, vector<int>  &, double *, int *, map<int, vector<double> > &); // not used (March 23, 2016)
-int     bootstrap_popstats_approximate_dist(vector<double> &, vector<double> &, vector<int>  &, double *, int *, int, map<int, vector<double> > &, map<int, vector<double> > &); // not used (March 23, 2016)
 double  bootstrap_approximate_pval(int, double, map<int, vector<double> > &);
 
 bool hap_compare(const pair<string,int>&, const pair<string,int>&);
